@@ -1,5 +1,5 @@
 /*
-  Copyright 2011 David Robillard <http://drobilla.net>
+  Copyright 2011-2015 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -20,72 +20,75 @@
 
 #include "serd/serd.h"
 
-#include "sord-config.h"
+#include "sord_config.h"
 #include "sord_internal.h"
 
-typedef struct {
+struct SordInserterImpl {
+	SordModel* model;
 	SerdEnv*   env;
-	SordNode*  graph_uri_node;
-	SordWorld* world;
-	SordModel* sord;
-} ReadState;
+};
 
-static SerdStatus
-event_base(void*           handle,
-           const SerdNode* uri_node)
+SordInserter*
+sord_inserter_new(SordModel* model,
+                  SerdEnv*   env)
 {
-	ReadState* const state = (ReadState*)handle;
-
-	return serd_env_set_base_uri(state->env, uri_node);
+	SordInserter* inserter = (SordInserter*)malloc(sizeof(SordInserter));
+	inserter->model = model;
+	inserter->env   = env;
+	return inserter;
 }
 
-static SerdStatus
-event_prefix(void*           handle,
-             const SerdNode* name,
-             const SerdNode* uri_node)
+void
+sord_inserter_free(SordInserter* inserter)
 {
-	ReadState* const state = (ReadState*)handle;
-
-	return serd_env_set_prefix(state->env, name, uri_node);
+	free(inserter);
 }
 
-static SerdStatus
-event_statement(void*              handle,
-                SerdStatementFlags flags,
-                const SerdNode*    graph,
-                const SerdNode*    subject,
-                const SerdNode*    predicate,
-                const SerdNode*    object,
-                const SerdNode*    object_datatype,
-                const SerdNode*    object_lang)
+SerdStatus
+sord_inserter_set_base_uri(SordInserter*   inserter,
+                           const SerdNode* uri)
 {
-	ReadState* const state = (ReadState*)handle;
+	return serd_env_set_base_uri(inserter->env, uri);
+}
 
-	SordNode* s = sord_node_from_serd_node(state->world, state->env,
-	                                       subject, NULL, NULL);
-	SordNode* p = sord_node_from_serd_node(state->world, state->env,
-	                                       predicate, NULL, NULL);
-	SordNode* o = sord_node_from_serd_node(
-		state->world, state->env, object, object_datatype, object_lang);
+SerdStatus
+sord_inserter_set_prefix(SordInserter*   inserter,
+                         const SerdNode* name,
+                         const SerdNode* uri)
+{
+	return serd_env_set_prefix(inserter->env, name, uri);
+}
 
-	SordNode* g = NULL;
-	if (state->graph_uri_node) {
-		assert(graph->type == SERD_NOTHING);
-		g = sord_node_copy(state->graph_uri_node);
-	} else {
-		g = (graph && graph->buf)
-			? sord_node_from_serd_node(state->world, state->env,
-			                           graph, NULL, NULL)
-			: NULL;
+SerdStatus
+sord_inserter_write_statement(SordInserter*      inserter,
+                              SerdStatementFlags flags,
+                              const SerdNode*    graph,
+                              const SerdNode*    subject,
+                              const SerdNode*    predicate,
+                              const SerdNode*    object,
+                              const SerdNode*    object_datatype,
+                              const SerdNode*    object_lang)
+{
+	SordWorld* world = sord_get_world(inserter->model);
+	SerdEnv*   env   = inserter->env;
+
+	SordNode* g = sord_node_from_serd_node(world, env, graph, NULL, NULL);
+	SordNode* s = sord_node_from_serd_node(world, env, subject, NULL, NULL);
+	SordNode* p = sord_node_from_serd_node(world, env, predicate, NULL, NULL);
+	SordNode* o = sord_node_from_serd_node(world, env, object,
+	                                       object_datatype, object_lang);
+
+	if (!s || !p || !o) {
+		return SERD_ERR_BAD_ARG;
 	}
 
 	const SordQuad tup = { s, p, o, g };
-	sord_add(state->sord, tup);
+	sord_add(inserter->model, tup);
 
-	sord_node_free(state->world, s);
-	sord_node_free(state->world, p);
-	sord_node_free(state->world, o);
-	sord_node_free(state->world, g);
+	sord_node_free(world, o);
+	sord_node_free(world, p);
+	sord_node_free(world, s);
+	sord_node_free(world, g);
 
 	return SERD_SUCCESS;
 }
@@ -97,21 +100,26 @@ sord_new_reader(SordModel* model,
                 SerdSyntax syntax,
                 SordNode*  graph)
 {
-	ReadState* state = malloc(sizeof(ReadState));
-	state->env            = env;
-	state->graph_uri_node = graph;
-	state->world          = sord_get_world(model);
-	state->sord           = model;
+	SordInserter* inserter = sord_inserter_new(model, env);
 
 	SerdReader* reader = serd_reader_new(
-		syntax, state, free,
-		event_base, event_prefix, event_statement, NULL);
+		syntax, inserter, (void (*)(void* ptr))sord_inserter_free,
+		(SerdBaseSink)sord_inserter_set_base_uri,
+		(SerdPrefixSink)sord_inserter_set_prefix,
+		(SerdStatementSink)sord_inserter_write_statement,
+		NULL);
+
+	if (graph) {
+		serd_reader_set_default_graph(reader, sord_node_to_serd_node(graph));
+	}
 
 	return reader;
 }
 
-static void
-write_statement(SordModel* sord, SerdWriter* writer, SordQuad tup,
+static SerdStatus
+write_statement(SordModel*         sord,
+                SerdWriter*        writer,
+                SordQuad           tup,
                 SerdStatementFlags flags)
 {
 	const SordNode* s  = tup[SORD_SUBJECT];
@@ -131,12 +139,12 @@ write_statement(SordModel* sord, SerdWriter* writer, SordQuad tup,
 		language.n_bytes = lang_len;
 		language.n_chars = lang_len;
 		language.buf     = (const uint8_t*)lang_str;
-	};
+	}
 
 	// TODO: Subject abbreviation
 
 	if (sord_node_is_inline_object(s) && !(flags & SERD_ANON_CONT)) {
-		return;
+		return SERD_SUCCESS;
 	}
 
 	SerdStatus st = SERD_SUCCESS;
@@ -152,10 +160,10 @@ write_statement(SordModel* sord, SerdWriter* writer, SordQuad tup,
 
 		if (!st && sub_iter) {
 			flags |= SERD_ANON_CONT;
-			for (; !sord_iter_end(sub_iter); sord_iter_next(sub_iter)) {
+			for (; !st && !sord_iter_end(sub_iter); sord_iter_next(sub_iter)) {
 				SordQuad sub_tup;
 				sord_iter_get(sub_iter, sub_tup);
-				write_statement(sord, writer, sub_tup, flags);
+				st = write_statement(sord, writer, sub_tup, flags);
 			}
 			sord_iter_free(sub_iter);
 			serd_writer_end_anon(writer, so);
@@ -165,11 +173,7 @@ write_statement(SordModel* sord, SerdWriter* writer, SordQuad tup,
 			writer, flags, NULL, ss, sp, so, sd, &language);
 	}
 
-	if (st) {
-		fprintf(stderr, "Failed to write statement (%s)\n",
-		        serd_strerror(st));
-		return;
-	}
+	return st;
 }
 
 bool
@@ -191,11 +195,13 @@ sord_write_iter(SordIter*   iter,
 	}
 
 	SordModel* model = (SordModel*)sord_iter_get_model(iter);
-	for (; !sord_iter_end(iter); sord_iter_next(iter)) {
+	SerdStatus st    = SERD_SUCCESS;
+	for (; !st && !sord_iter_end(iter); sord_iter_next(iter)) {
 		SordQuad tup;
 		sord_iter_get(iter, tup);
-		write_statement(model, writer, tup, 0);
+		st = write_statement(model, writer, tup, 0);
 	}
 	sord_iter_free(iter);
-	return true;
+
+	return !st;
 }
